"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rotateR = exports.rotateL = exports.remove = exports.balance = void 0;
const binaryTreeUtils_1 = require("./binaryTreeUtils");
/**
 * @internal
 */
function balance(node) {
    if (node.balanceFactor > 1) {
        if (node.right.balanceFactor < 0) {
            node.right = rotateR(node.right);
        }
        node = rotateL(node);
    }
    else if (node.balanceFactor < -1) {
        if (node.left.balanceFactor > 0) {
            node.left = rotateL(node.left);
        }
        node = rotateR(node);
    }
    return node;
}
exports.balance = balance;
/**
 * @internal
 */
function remove(stack) {
    let edge = stack.value;
    let node = edge.to;
    // If not found
    if (node == null) {
        return false;
    }
    // Find the replacement
    if (node.right == null) {
        // If no right child, replace with left
        node = node.left;
    }
    else if (node.left == null) {
        // If no left child, replace with right
        node = node.right;
    }
    else {
        // If two children, find the successor
        stack = binaryTreeUtils_1.successorStack(stack);
        edge = stack.value;
        // Swap with the successor
        const temp = edge.to;
        node.value = temp.value;
        // Replace with the successor's right child
        node = temp.right;
    }
    // Make the replacement / update the tree
    let label = edge.label;
    edge.from[label] = edge.to = node;
    // Balance the tree
    while (stack.next) {
        stack = stack.next;
        edge = stack.value;
        edge.to.balanceFactor -= label === 'left' ? -1 : 1;
        edge.to = balance(edge.to);
        edge.from[(label = edge.label)] = edge.to;
        if (edge.to.balanceFactor !== 0) {
            break;
        }
    }
    return true;
}
exports.remove = remove;
/**
 * @internal
 */
function rotateL(P) {
    const R = P.right;
    P.right = R.left;
    R.left = P;
    P.balanceFactor -= 1 + Math.max(0, R.balanceFactor);
    R.balanceFactor -= 1 - Math.min(0, P.balanceFactor);
    return R;
}
exports.rotateL = rotateL;
/**
 * @internal
 */
function rotateR(P) {
    const L = P.left;
    P.left = L.right;
    L.right = P;
    P.balanceFactor += 1 - Math.min(0, L.balanceFactor);
    L.balanceFactor += 1 + Math.max(0, P.balanceFactor);
    return L;
}
exports.rotateR = rotateR;
//# sourceMappingURL=avlTreeUtils.js.map